@@ -8,6 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 /* 
@@ -46,6 +47,15 @@
  *
  * action=mount_add&mount=/live&server=myserver.com&port=8000
  * action=mount_remove&mount=/live&server=myserver.com&port=8000
+ *
+ * On source client connection, a request can be made to trigger a URL request
+ * to verify the details externally. Post info is
+ *
+ * action=stream_auth&mount=/stream&ip=IP&server=SERVER&port=8000&user=fred&pass=pass
+ *
+ * As admin requests can come in for a stream (eg metadata update) these requests
+ * can be issued while stream is active. For these &admin=1 is added to the POST
+ * details.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,10 +86,13 @@
 #define CATMODULE "auth_url"
 
 typedef struct {
+    char *pass_headers; // headers passed from client to addurl.
+    char *prefix_headers; // prefix for passed headers.
     char *addurl;
     char *removeurl;
     char *stream_start;
     char *stream_end;
+    char *stream_auth;
     char *username;
     char *password;
     char *auth_header;
@@ -96,12 +109,14 @@ static void auth_url_clear(auth_t *self)
 {
     auth_url *url;
 
-    INFO0 ("Doing auth URL cleanup");
+    ICECAST_LOG_INFO("Doing auth URL cleanup");
     url = self->state;
     self->state = NULL;
     curl_easy_cleanup (url->handle);
     free (url->username);
     free (url->password);
+    free (url->pass_headers);
+    free (url->prefix_headers);
     free (url->removeurl);
     free (url->addurl);
     free (url->stream_start);
@@ -123,7 +138,7 @@ static int my_getpass(void *client, char *prompt, char *buffer, int buflen)
 #endif
 
 
-static int handle_returned_header (void *ptr, size_t size, size_t nmemb, void *stream)
+static size_t handle_returned_header (void *ptr, size_t size, size_t nmemb, void *stream)
 {
     auth_client *auth_user = stream;
     unsigned bytes = size * nmemb;
@@ -157,7 +172,7 @@ static int handle_returned_header (void *ptr, size_t size, size_t nmemb, void *s
 }
 
 /* capture returned data, but don't do anything with it */
-static int handle_returned_data (void *ptr, size_t size, size_t nmemb, void *stream)
+static size_t handle_returned_data (void *ptr, size_t size, size_t nmemb, void *stream)
 {
     return (int)(size*nmemb);
 }
@@ -174,14 +189,23 @@ static auth_result url_remove_listener (auth_client *auth_user)
     ice_config_t *config;
     int port;
     char *userpwd = NULL, post [4096];
+    const char *agent;
+    char *user_agent, *ipaddr;
 
     if (url->removeurl == NULL)
         return AUTH_OK;
+
     config = config_get_config ();
     server = util_url_escape (config->hostname);
     port = config->port;
     config_release_config ();
 
+    agent = httpp_getvar (client->parser, "user-agent");
+    if (agent)
+        user_agent = util_url_escape (agent);
+    else
+        user_agent = strdup ("-");
+
     if (client->username)
         username = util_url_escape (client->username);
     else
@@ -197,16 +221,19 @@ static auth_result url_remove_listener (auth_client *auth_user)
     if (mountreq == NULL)
         mountreq = httpp_getvar (client->parser, HTTPP_VAR_URI);
     mount = util_url_escape (mountreq);
+    ipaddr = util_url_escape (client->con->ip);
 
     snprintf (post, sizeof (post),
             "action=listener_remove&server=%s&port=%d&client=%lu&mount=%s"
-            "&user=%s&pass=%s&duration=%lu",
+            "&user=%s&pass=%s&duration=%lu&ip=%s&agent=%s",
             server, port, client->con->id, mount, username,
-            password, (long unsigned)duration);
+            password, (long unsigned)duration, ipaddr, user_agent);
     free (server);
     free (mount);
     free (username);
     free (password);
+    free (ipaddr);
+    free (user_agent);
 
     if (strchr (url->removeurl, '@') == NULL)
     {
@@ -217,7 +244,7 @@ static auth_result url_remove_listener (auth_client *auth_user)
             /* auth'd requests may not have a user/pass, but may use query args */
             if (client->username && client->password)
             {
-                int len = strlen (client->username) + strlen (client->password) + 2;
+                size_t len = strlen (client->username) + strlen (client->password) + 2;
                 userpwd = malloc (len);
                 snprintf (userpwd, len, "%s:%s", client->username, client->password);
                 curl_easy_setopt (url->handle, CURLOPT_USERPWD, userpwd);
@@ -236,7 +263,7 @@ static auth_result url_remove_listener (auth_client *auth_user)
     curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
 
     if (curl_easy_perform (url->handle))
-        WARN2 ("auth to server %s failed with %s", url->removeurl, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", url->removeurl, url->errormsg);
 
     free (userpwd);
 
@@ -256,6 +283,10 @@ static auth_result url_add_listener (auth_client *auth_user)
     char *mount, *ipaddr, *server;
     ice_config_t *config;
     char *userpwd = NULL, post [4096];
+    ssize_t post_offset;
+    char *pass_headers, *cur_header, *next_header;
+    const char *header_val;
+    char *header_valesc;
 
     if (url->addurl == NULL)
         return AUTH_OK;
@@ -264,16 +295,20 @@ static auth_result url_add_listener (auth_client *auth_user)
     server = util_url_escape (config->hostname);
     port = config->port;
     config_release_config ();
+
     agent = httpp_getvar (client->parser, "user-agent");
-    if (agent == NULL)
-        agent = "-";
-    user_agent = util_url_escape (agent);
+    if (agent)
+        user_agent = util_url_escape (agent);
+    else
+        user_agent = strdup ("-");
+
     if (client->username)
-        username  = util_url_escape (client->username);
+        username = util_url_escape (client->username);
     else
         username = strdup ("");
+
     if (client->password)
-        password  = util_url_escape (client->password);
+        password = util_url_escape (client->password);
     else
         password = strdup ("");
 
@@ -284,7 +319,7 @@ static auth_result url_add_listener (auth_client *auth_user)
     mount = util_url_escape (mountreq);
     ipaddr = util_url_escape (client->con->ip);
 
-    snprintf (post, sizeof (post),
+    post_offset = snprintf (post, sizeof (post),
             "action=listener_add&server=%s&port=%d&client=%lu&mount=%s"
             "&user=%s&pass=%s&ip=%s&agent=%s",
             server, port, client->con->id, mount, username,
@@ -296,6 +331,35 @@ static auth_result url_add_listener (auth_client *auth_user)
     free (password);
     free (ipaddr);
 
+    pass_headers = NULL;
+    if (url->pass_headers)
+        pass_headers = strdup (url->pass_headers);
+    if (pass_headers)
+    {
+        cur_header = pass_headers;
+        while (cur_header)
+        {
+	    next_header = strstr (cur_header, ",");
+	    if (next_header)
+	    {
+		*next_header=0;
+                next_header++;
+	    }
+
+            header_val = httpp_getvar (client->parser, cur_header);
+            if (header_val)
+            {
+                header_valesc = util_url_escape (header_val);
+                post_offset += snprintf (post+post_offset, sizeof (post)-post_offset, "&%s%s=%s",
+                                         url->prefix_headers ? url->prefix_headers : "",
+                                         cur_header, header_valesc);
+                free (header_valesc);
+            }
+
+	    cur_header = next_header;
+        }
+    }
+
     if (strchr (url->addurl, '@') == NULL)
     {
         if (url->userpwd)
@@ -305,7 +369,7 @@ static auth_result url_add_listener (auth_client *auth_user)
             /* auth'd requests may not have a user/pass, but may use query args */
             if (client->username && client->password)
             {
-                int len = strlen (client->username) + strlen (client->password) + 2;
+                size_t len = strlen (client->username) + strlen (client->password) + 2;
                 userpwd = malloc (len);
                 snprintf (userpwd, len, "%s:%s", client->username, client->password);
                 curl_easy_setopt (url->handle, CURLOPT_USERPWD, userpwd);
@@ -330,13 +394,13 @@ static auth_result url_add_listener (auth_client *auth_user)
 
     if (res)
     {
-        WARN2 ("auth to server %s failed with %s", url->addurl, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", url->addurl, url->errormsg);
         return AUTH_FAILED;
     }
     /* we received a response, lets see what it is */
     if (client->authenticated)
         return AUTH_OK;
-    INFO2 ("client auth (%s) failed with \"%s\"", url->addurl, url->errormsg);
+    ICECAST_LOG_INFO("client auth (%s) failed with \"%s\"", url->addurl, url->errormsg);
     return AUTH_FAILED;
 }
 
@@ -348,7 +412,7 @@ static void url_stream_start (auth_client *auth_user)
 {
     char *mount, *server;
     ice_config_t *config = config_get_config ();
-    mount_proxy *mountinfo = config_find_mount (config, auth_user->mount);
+    mount_proxy *mountinfo = config_find_mount (config, auth_user->mount, MOUNT_TYPE_NORMAL);
     auth_t *auth = mountinfo->auth;
     auth_url *url = auth->state;
     char *stream_start_url;
@@ -388,7 +452,7 @@ static void url_stream_start (auth_client *auth_user)
     curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
 
     if (curl_easy_perform (url->handle))
-        WARN2 ("auth to server %s failed with %s", stream_start_url, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", stream_start_url, url->errormsg);
 
     auth_release (auth);
     free (stream_start_url);
@@ -400,7 +464,7 @@ static void url_stream_end (auth_client *auth_user)
 {
     char *mount, *server;
     ice_config_t *config = config_get_config ();
-    mount_proxy *mountinfo = config_find_mount (config, auth_user->mount);
+    mount_proxy *mountinfo = config_find_mount (config, auth_user->mount, MOUNT_TYPE_NORMAL);
     auth_t *auth = mountinfo->auth;
     auth_url *url = auth->state;
     char *stream_end_url;
@@ -440,13 +504,59 @@ static void url_stream_end (auth_client *auth_user)
     curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
 
     if (curl_easy_perform (url->handle))
-        WARN2 ("auth to server %s failed with %s", stream_end_url, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", stream_end_url, url->errormsg);
 
     auth_release (auth);
     free (stream_end_url);
     return;
 }
 
+static void url_stream_auth (auth_client *auth_user)
+{
+    ice_config_t *config;
+    int port;
+    client_t *client = auth_user->client;
+    auth_url *url = client->auth->state;
+    char *mount, *host, *user, *pass, *ipaddr, *admin="";
+    char post [4096];
+
+    if (strchr (url->stream_auth, '@') == NULL)
+    {
+        if (url->userpwd)
+            curl_easy_setopt (url->handle, CURLOPT_USERPWD, url->userpwd);
+        else
+            curl_easy_setopt (url->handle, CURLOPT_USERPWD, "");
+    }
+    else
+        curl_easy_setopt (url->handle, CURLOPT_USERPWD, "");
+    curl_easy_setopt (url->handle, CURLOPT_URL, url->stream_auth);
+    curl_easy_setopt (url->handle, CURLOPT_POSTFIELDS, post);
+    curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
+    if (strcmp (auth_user->mount, httpp_getvar (client->parser, HTTPP_VAR_URI)) != 0)
+        admin = "&admin=1";
+    mount = util_url_escape (auth_user->mount);
+    config = config_get_config ();
+    host = util_url_escape (config->hostname);
+    port = config->port;
+    config_release_config ();
+    user = util_url_escape (client->username);
+    pass = util_url_escape (client->password);
+    ipaddr = util_url_escape (client->con->ip);
+
+    snprintf (post, sizeof (post),
+            "action=stream_auth&mount=%s&ip=%s&server=%s&port=%d&user=%s&pass=%s%s",
+            mount, ipaddr, host, port, user, pass, admin);
+    free (ipaddr);
+    free (user);
+    free (pass);
+    free (mount);
+    free (host);
+
+    client->authenticated = 0;
+    if (curl_easy_perform (url->handle))
+        ICECAST_LOG_WARN("auth to server %s failed with %s", url->stream_auth, url->errormsg);
+}
+
 
 static auth_result auth_url_adduser(auth_t *auth, const char *username, const char *password)
 {
@@ -479,6 +589,9 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
     url_info->auth_header = strdup ("icecast-auth-user: 1\r\n");
     url_info->timelimit_header = strdup ("icecast-auth-timelimit:");
 
+    /* force auth thread to call function. this makes sure the auth_t is attached to client */
+    authenticator->authenticate = url_add_listener;
+
     while(options) {
         if(!strcmp(options->name, "username"))
         {
@@ -490,9 +603,18 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
             free (url_info->password);
             url_info->password = strdup (options->value);
         }
+        if(!strcmp(options->name, "headers"))
+        {
+            free (url_info->pass_headers);
+            url_info->pass_headers = strdup (options->value);
+        }
+        if(!strcmp(options->name, "header_prefix"))
+        {
+            free (url_info->prefix_headers);
+            url_info->prefix_headers = strdup (options->value);
+        }
         if(!strcmp(options->name, "listener_add"))
         {
-            authenticator->authenticate = url_add_listener;
             free (url_info->addurl);
             url_info->addurl = strdup (options->value);
         }
@@ -514,6 +636,12 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
             free (url_info->stream_end);
             url_info->stream_end = strdup (options->value);
         }
+        if(!strcmp(options->name, "stream_auth"))
+        {
+            authenticator->stream_auth = url_stream_auth;
+            free (url_info->stream_auth);
+            url_info->stream_auth = strdup (options->value);
+        }
         if(!strcmp(options->name, "auth_header"))
         {
             free (url_info->auth_header);
@@ -555,7 +683,7 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
         snprintf (url_info->userpwd, len, "%s:%s", url_info->username, url_info->password);
     }
 
-    INFO0("URL based authentication setup");
+    ICECAST_LOG_INFO("URL based authentication setup");
     return 0;
 }
 
