@@ -3,12 +3,11 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org,
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org, 
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
- * Copyright 2014-2018, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 
@@ -21,8 +20,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ogg/ogg.h>
+#ifdef HAVE_KATE
+#include <kate/oggkate.h>
+#endif
+
+typedef struct source_tag source_t;
 
-#include "compat.h"
 #include "refbuf.h"
 #include "format_ogg.h"
 #include "format_kate.h"
@@ -36,7 +39,12 @@
 typedef struct _kate_codec_tag
 {
     int             headers_done;
-    size_t          num_headers;
+#ifdef HAVE_KATE
+    kate_info       ki;
+    kate_comment    kc;
+#endif
+    int             num_headers;
+    int             granule_shift;
     ogg_int64_t     last_iframe;
     ogg_int64_t     prev_granulepos;
 } kate_codec_t;
@@ -53,6 +61,10 @@ static void kate_codec_free (ogg_state_t *ogg_info, ogg_codec_t *codec)
     stats_event (ogg_info->mount, "frame_rate", NULL);
     stats_event (ogg_info->mount, "frame_size", NULL);
     */
+#ifdef HAVE_KATE
+    kate_info_clear (&kate->ki);
+    kate_comment_clear (&kate->kc);
+#endif
     ogg_stream_clear (&codec->os);
     free (kate);
     free (codec);
@@ -62,7 +74,7 @@ static void kate_codec_free (ogg_state_t *ogg_info, ogg_codec_t *codec)
 /* kate pages are not rebuilt, so here we just for headers and then
  * pass them straight through to the the queue
  */
-static refbuf_t *process_kate_page (ogg_state_t *ogg_info, ogg_codec_t *codec, ogg_page *page, format_plugin_t *plugin)
+static refbuf_t *process_kate_page (ogg_state_t *ogg_info, ogg_codec_t *codec, ogg_page *page)
 {
     kate_codec_t *kate = codec->specific;
     ogg_packet packet;
@@ -75,13 +87,42 @@ static refbuf_t *process_kate_page (ogg_state_t *ogg_info, ogg_codec_t *codec, o
         ogg_info->error = 1;
         return NULL;
     }
-    granulepos = ogg_page_granulepos(page);
+    granulepos = ogg_page_granulepos (page);
 
     while (ogg_stream_packetout (&codec->os, &packet) > 0)
     {
         if (!kate->headers_done)
         {
-            header_page = (packet.bytes > 0 && (packet.packet[0] & 0x80));
+#ifdef HAVE_KATE
+            int ret = kate_ogg_decode_headerin (&kate->ki, &kate->kc, &packet);
+            if (ret < 0)
+            {
+                ogg_info->error = 1;
+                ICECAST_LOG_WARN("problem with kate header");
+                return NULL;
+            }
+            header_page = 1;
+            kate->num_headers = kate->ki.num_headers;
+            codec->headers++;
+            if (ret > 0)
+            {
+                kate->headers_done = 1;
+                /* TODO: what to replace this with ?
+                ogg_info->bitrate += theora->ti.target_bitrate;
+                stats_event_args (ogg_info->mount, "video_bitrate", "%ld",
+                        (long)theora->ti.target_bitrate);
+                stats_event_args (ogg_info->mount, "video_quality", "%ld",
+                        (long)theora->ti.quality);
+                stats_event_args (ogg_info->mount, "frame_size", "%ld x %ld",
+                        (long)theora->ti.frame_width,
+                        (long)theora->ti.frame_height);
+                stats_event_args (ogg_info->mount, "frame_rate", "%.2f",
+                        (float)theora->ti.fps_numerator/theora->ti.fps_denominator);
+                */
+            }
+            continue;
+#else
+            header_page = (packet.bytes>0 && (packet.packet[0]&0x80));
             if (!header_page)
                 break;
             codec->headers++;
@@ -92,6 +133,7 @@ static refbuf_t *process_kate_page (ogg_state_t *ogg_info, ogg_codec_t *codec, o
                 kate->num_headers = packet.packet[11];
             }
             continue;
+#endif
         }
 
         if (codec->headers < kate->num_headers)
@@ -136,20 +178,36 @@ static refbuf_t *process_kate_page (ogg_state_t *ogg_info, ogg_codec_t *codec, o
 /* Check if specified BOS page is the start of a kate stream and
  * if so, create a codec structure for handling it
  */
-ogg_codec_t *initial_kate_page(format_plugin_t *plugin, ogg_page *page)
+ogg_codec_t *initial_kate_page (format_plugin_t *plugin, ogg_page *page)
 {
     ogg_state_t *ogg_info = plugin->_state;
-    ogg_codec_t *codec = calloc(1, sizeof(ogg_codec_t));
+    ogg_codec_t *codec = calloc (1, sizeof (ogg_codec_t));
     ogg_packet packet;
 
-    kate_codec_t *kate_codec = calloc(1, sizeof(kate_codec_t));
+    kate_codec_t *kate_codec = calloc (1, sizeof (kate_codec_t));
 
-    ogg_stream_init(&codec->os, ogg_page_serialno(page));
-    ogg_stream_pagein(&codec->os, page);
+    ogg_stream_init (&codec->os, ogg_page_serialno (page));
+    ogg_stream_pagein (&codec->os, page);
 
-    ogg_stream_packetout(&codec->os, &packet);
+#ifdef HAVE_KATE
+    kate_info_init (&kate_codec->ki);
+    kate_comment_init (&kate_codec->kc);
+#endif
+
+    ogg_stream_packetout (&codec->os, &packet);
 
     ICECAST_LOG_DEBUG("checking for kate codec");
+#ifdef HAVE_KATE
+    if (kate_ogg_decode_headerin (&kate_codec->ki, &kate_codec->kc, &packet) < 0)
+    {
+        kate_info_clear (&kate_codec->ki);
+        kate_comment_clear (&kate_codec->kc);
+        ogg_stream_clear (&codec->os);
+        free (kate_codec);
+        free (codec);
+        return NULL;
+    }
+#else
     /* we don't have libkate, so we examine the packet magic by hand */
     if ((packet.bytes<9) || memcmp(packet.packet, "\x80kate\0\0\0\0", 9))
     {
@@ -158,6 +216,7 @@ ogg_codec_t *initial_kate_page(format_plugin_t *plugin, ogg_page *page)
         free (codec);
         return NULL;
     }
+#endif
 
     ICECAST_LOG_INFO("seen initial kate header");
     codec->specific = kate_codec;
@@ -166,7 +225,7 @@ ogg_codec_t *initial_kate_page(format_plugin_t *plugin, ogg_page *page)
     codec->headers = 1;
     codec->name = "Kate";
 
-    format_ogg_attach_header(ogg_info, page);
+    format_ogg_attach_header (ogg_info, page);
     ogg_info->codec_sync = codec;
     return codec;
 }
